use strict;

use ExtUtils::MakeMaker qw(prompt);
use Module::Build;

my $class = Module::Build->subclass( class => 'Apprentice::Builder',
				     code => <<'EOF',
use strict;
use Digest::SHA1;
use File::Basename;
use File::Copy;
use File::Path;
use File::Spec;

sub VERSION { '0.15' }

sub ACTION_install
{
    my $self = shift;

    # need to do this before trying to substitute db name
    $self->ACTION_build;

    $self->_substitute_db_params;

    $self->_make_server_secret;

    $self->SUPER::ACTION_install(@_);

    $self->install_htdocs;
    $self->copy_alzabo_schema;

    $self->setup_db;
}

sub _substitute_db_params
{
    my $self = shift;

    my $fh = do { local *FH; *FH; };

    open $fh, "<./blib/lib/Apprentice/Data.pm"
	or die "cannot read ./blib/lib/Apprentice/Data.pm: $!";

    my @module = <$fh>;

    close $fh;

    open $fh, ">./blib/lib/Apprentice/Data.pm"
	or die "cannot write to ./blib/lib/Apprentice/Data.pm: $!";

    foreach my $line (@module)
    {
	foreach ( qw( DB_USER DB_PASSWORD DB_HOST DB_NAME ) )
	{
	    $line =~
		s/!!$_!!/
		 $self->{properties}{lc $_} ? "'$self->{properties}{lc $_}'" : 'undef'/eg;
	}
	
	print $fh $line;
    }

    close $fh;
}

sub _make_server_secret
{
    my $self = shift;

    my $fh = do { local *FH; *FH; };

    open $fh, "<./blib/lib/Apprentice.pm"
	or die "cannot read ./blib/lib/Apprentice.pm: $!";

    my @module = <$fh>;

    close $fh;

    open $fh, ">./blib/lib/Apprentice.pm"
	or die "cannot write to ./blib/lib/Apprentice.pm: $!";

    my $secret = Digest::SHA1::sha1_base64( $$ . rand( 1_000_000 ) . time );
    foreach my $line (@module)
    {
	$line =~ s/!!SECRET!!/$secret/;

	print $fh $line;
    }

    close $fh;
}

sub ACTION_install_htdocs
{
    my $self = shift;

    $self->install_htdocs;
}

sub install_htdocs
{
    my $self = shift;

    my $target = $self->{properties}{htdocs_target};

    my $files =
        $self->rscan_dir( './htdocs',
			  qr/(?:handler|\.mas|\.html|\.css|\.js|\.jpg|\.gif|\.png)$/ );

    $self->_copy_files( $files, $target, './htdocs/' );
}

sub copy_alzabo_schema
{
    my $self = shift;

    eval { require Alzabo::Config };
    if ($@)
    {
	die "Unable to load Alzabo::Config.  Is Alzabo installed? ($@)";
    }

    my $base = Alzabo::Config::schema_dir();
    my $target = File::Spec->catdir( $base, 'apprentice' );

    my $files = $self->rscan_dir( './alzabo', qr/apprentice/ );

    $self->_copy_files( $files, $target, './alzabo/' );
}

sub _copy_files
{
    my ($self, $files, $target, $strip) = @_;

    foreach my $file (@$files)
    {
	my $stripped = $file;
	$stripped =~ s,^$strip,, if $strip;

	my $to = File::Spec->catfile( $target, $stripped );

	print "$file -> $to\n";

	File::Path::mkpath( File::Basename::dirname($to), 1, 0755 );

	File::Copy::copy( $file, $to );
    }
}

sub setup_db
{
    my $self = shift;

    $self->_sync_schema;

    $self->_insert_db_defaults;
}

sub _sync_schema
{
    my $self = shift;

    print "Syncing your RDBMS to the latest version of the apprentice schema...\n";

    require Alzabo::Create::Schema;
    my $schema = Alzabo::Create::Schema->load_from_file( name => 'apprentice' );

    if ( $self->{properties}{db_name} ne 'apprentice' )
    {
	$schema = $schema->clone( name => $self->{properties}{db_name} );
    }

    $schema->sync_backend( $self->_db_params );

    $schema->save_to_file;
}

my %defaults = ( Category => [ { name => 'Database' },
			       { name => 'Bio-informatics' },
			       { name => 'Web' },
			       { name => 'XML' },
			       { name => 'mod_perl' },
			       { name => 'System administration' },
			       { name => 'GUI apps' },
			     ],

		 Role => [ { role => 'Mentor' },
			   { role => 'Apprentice' } ],

		 ProjectStatus => [ { status => 'Idea' },
				    { status => 'Active' } ],

		 ProjectSupportLevel => [ { support_level => 'A lot' },
				          { support_level => 'Some' },
				          { support_level => 'Little' } ],

		 UserStatus => [ { status => 'Available' },
				 { status => 'Semi-available' },
				 { status => 'Busy' } ],

	       );

sub _insert_db_defaults
{
    my $self = shift;

    print "Inserting various default values into the apprentice database...\n";

    require Alzabo::Runtime::Schema;
    my $schema =
	Alzabo::Runtime::Schema->load_from_file( name => $self->{properties}{db_name} );

    $schema->connect( $self->_db_params );

    foreach my $table_name ( keys %defaults )
    {
	my $table = $schema->table($table_name);

    ENTRY:
	foreach my $entry ( @{ $defaults{$table_name} } )
	{
	    next ENTRY if
                $table->rows_where( where =>
                                    [ map { [ $table->column($_), '=', $entry->{$_} ] }
                                      keys %$entry ] )->next;

            $table->insert( values => $entry );
	}
    }

    unless ( $schema->table('User')->row_count )
    {
	print "Creating default admin username with password 'fayewong'...\n";

	$schema->table('User')->insert
	    ( values => { username  => 'admin',
			  password  => 'fayewong',
			  real_name => 'Faye Wong',
			  email_address => 'faye@wong',
			  user_status_id => undef,
			  is_admin => 1,
		      },
	      );
    }
}

sub _db_params
{
    my $self = shift;

    return ( user => $self->{properties}{db_user},
	     password => $self->{properties}{db_password},
	     host => $self->{properties}{db_host} );
}
EOF
				     );

print <<'EOF';

Welcome to the installer for the Perl Apprentice site.  In an effort
to make this code accessible for playing around with, we have an
honest-to-dog command-line installer which asks you questions and
stuff.  So without further ado ...

EOF

my %opts = get_user_settings() unless $ARGV[0] eq '--dist' && shift @ARGV;

my $build = $class->new( module_name => 'Apprentice',
                         %opts,
			 prereq => { Alzabo => 0.67,
				     'Apache::Request' => 0.33,
				     'Digest::SHA1' => 0,
                                     'Email::Valid' => 0,
                                     'HTML::FromText' => 0,
                                     'Lingua::EN::Inflect' => 0,
				     'MIME::Lite' => 0,
				     'Time::Piece' => 1.0,
				     URI => 0,
				     'URI::Escape' => 0,
				   },
                         license => 'perl',
		       );

$build->create_build_script;

sub get_user_settings
{
    my %s;

    $s{htdocs_target} = get_htdocs_target();
    @s{ qw( db_user db_password db_host db_name ) } = get_db_params();

    return %s;
}

sub get_htdocs_target
{
    my $target;

    until (defined $target && $target ne '')
    {
	$target = prompt( "What is your web server's document root\n(do not use '~' in the path name)?" );
    }

    return $target;
}

sub get_db_params
{
    print <<'EOF';

We need to set up a new MySQL database and insert some defaults.  In
order to do this, we need to know how to connect.

EOF

    my $user;
    my $password;
    until ( defined $user )
    {
	$user = prompt( "What user should be used to connect to the RDBMS\n(this user must be able to create a new database\n or access an existing, empty one)?" );
    }

    if ( $user ne '' )
    {
	until ( defined $password )
	{
	    $password = prompt( "\nWhat password should be used with the $user user?" );
	}
    }

    my $host =
	prompt( "\nWhat host is the RDBMS on (hit enter if no host\nparameter is needed to connect)?" );

    my $db_name =
	prompt( "\nWhat should be the name of the database created (or used)?", "apprentice" );

    return ($user, $password, $host, $db_name);
}
